//********************************************************************
//*** ResourcePool/NoopCommandWrapper.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: NoopCommandWrapper.hh,v 1.6 2009-05-06 17:30:35 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_NOOPCOMMANDWRAPPER_HH
#define RESOURCEPOOL_NOOPCOMMANDWRAPPER_HH

#include "mm/gc_ptr.hh"
#include "util/Clone.hh"

namespace fatalmind {

/**
 * This class takes another Command as argument and 
 * does not execute it.
 */

template<class CommandType>
class NoopCommandWrapper: public CommandType
{
	protected:
	private:
		gc_ptr<CommandType> m;

		NoopCommandWrapper(const NoopCommandWrapper& rho) 
		: CommandType(rho)
		, m(rho.m)
		{

		}
	public:
		NoopCommandWrapper(const CommandType& a_m) 
		: CommandType(a_m)
		, m(a_m.template clone<CommandType>())
		{
		};
		virtual ~NoopCommandWrapper() {
		}

		virtual void execute(typename CommandType::pool_type::resource_t& r) {
			// a real noop does not execute m :)
		}

		const CommandType& getCommandRef() const {
			return *m.get();
		}

		virtual void outputoperator(std::ostream& s) const {
			s << "NoopCommandWrapper[" << (*m) << ']';
		}
	protected:
		virtual Clone* DoClone() const {
			return new NoopCommandWrapper(*this);
		}

}; // class NoopCommandWrapper

} // namespace fatalmind
#endif
