//********************************************************************
//*** ResourcePool/SQL/Bind.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: Bind.cc,v 1.19 2009-03-14 11:35:12 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <string>
#include <ostream>

#include "Bind.hh"

namespace fatalmind {
namespace SQL {

template<class VBC, template<class> class CT>
void Bind<VBC, CT>::bindPtr(const unsigned int pos, gc_ptr<VBC>& ptr) {
	register unsigned int s = _args.size();
	if (s == pos) { // to be inserted at the end
		_args.push_back(ptr);
	} else {
		if (s < pos) { // in bind with hole, one or more positions are missing
			_args.resize(pos);
			_args.push_back(ptr);
		} else {
			_args[pos] = ptr;
		}
	}
}

template<class VBC, template<class> class CT>
void Bind<VBC, CT>::clearParameters() {
	_args.clear();
}
template<class VBC, template<class> class CT>
void Bind<VBC, CT>::bindoutputoperator(std::ostream& s) const {
	bindoutputoperator(s, _args);
}

template<class VBC, template<class> class CT>
void Bind<VBC, CT>::bindoutputoperator(std::ostream& s, const _argst& args) {
	s << "Bind[";	
	typename _argst::const_iterator end = args.end();
	int idx = 0;
	for (typename _argst::const_iterator i = args.begin(); i != end; ++i, ++ idx) {
		if (idx > 0) {
			s << "; ";
		}
		s << '(' << idx << ")";
		if (i->get() == 0) {
			s << "=(NotSet)";
		} else if ((*i)->isnull()) {
			s << " is NULL";
		} else {
			s << '=';
			(*i)->outputoperator(s);
		}

	}
	s << ']';
}

} // namespace SQL
} // namespace fatalmind
