//********************************************************************
//*** ResourcePool/SQL/BindInterface.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: BindInterface.hh,v 1.9 2009-05-02 22:17:15 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_BINDINTERFACE_HH
#define RESOURCEPOOL_SQL_BINDINTERFACE_HH

namespace fatalmind {
namespace SQL {

/**
 * This class is for documentation purposes only.
 * It's not possible to use this polimorphycally since
 * the methods are not virtual (and can not be virtual since
 * it is a member template).
 */
class BindInterface {
	public:
		/**
		 * Binds an actual input value for an placeholder in the 
		 * SQL statement.
		 *
		 * All binding is done by value! This means the passed data
		 * are copied to internal storage. Changing your original
		 * copy of your data after passing it to bind() is therefore
		 * safe.
		 * 
		 * \param pos The position indicator. The number of 
		 *        placeholder (?) in the SQL string. Counted from the 
		 *        left starting at 0.
		 * \param val the actual value.
		 * \param nul NULL inidcator, if set to yes the actual value 
		 *        to be bound will be NULL. Please note that you still
		 *        need to provide a valid value!
		 */
		virtual void bind(const unsigned int pos, const char& val, bool isnull = false) = 0;
		virtual void bind(const unsigned int pos, const signed char& val, bool isnull = false) = 0;
		virtual void bind(const unsigned int pos, const unsigned char& val, bool isnull = false) = 0;

		virtual void bind(const unsigned int pos, const short int& val, bool isnull = false) = 0;
		virtual void bind(const unsigned int pos, const unsigned short int& val, bool isnull = false) = 0;

		virtual void bind(const unsigned int pos, const int& val, bool isnull = false) = 0;
		virtual void bind(const unsigned int pos, const unsigned int& val, bool isnull = false) = 0;

		virtual void bind(const unsigned int pos, const long int& val, bool isnull = false) = 0;
		virtual void bind(const unsigned int pos, const unsigned long int& val, bool isnull = false) = 0;

		virtual void bind(const unsigned int pos, const float& val, bool isnull = false) = 0;
		virtual void bind(const unsigned int pos, const double& val, bool isnull = false) = 0;
		virtual void bind(const unsigned int pos, const long double& val, bool isnull = false) = 0;

		virtual void bind(const unsigned int pos, const std::string& val, bool isnull = false) = 0;

		/*
		 * Removes all currently bound values.
		 * A newly constructed object is always cleared.
		 */
		virtual void clearParameters() = 0;
	protected:
		virtual ~BindInterface() {
		}
};

} // namespace SQL
} // namespace fatalmind
#endif
