//********************************************************************
//*** ResourcePool/SQL/Factory.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: Factory.hh,v 1.12 2009-03-02 14:16:44 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_FACTORY_HH
#define RESOURCEPOOL_SQL_FACTORY_HH

#include "mm/gc_ptr.hh"
#include "Thread/Lock.hh"

namespace fatalmind {
namespace SQL {

template<class TM = DefaultThreadedModel>
class SQLFactory;

}
}

#include "ResourcePool/SQL/Resource.hh"
#include "ResourcePool/SQL/SQLFactoryInterface.hh"
#include "ResourcePool/ResourcePoolStreamLogger.hh"

namespace fatalmind {
namespace SQL {

template<class TM> class SQLFactory_Impl;

template<class TM> 
class SQLFactory : public SQLFactoryInterface {
		typedef SQLFactoryInterface super;
	public:
		/**
		 * \param logon type:[logon]
		 */
		SQLFactory(const std::string& logon, ResourcePoolEventObserver& e = ResourcePoolStreamLogger::defaultLogger);
		SQLFactory(const SQLFactory&);
		~SQLFactory() throw();

		DummySQLResource<TM>* create_resource() const;

		virtual databaseType_t getDatabaseType() const;
		
		// the real observer is not the one in the super class,
		// it's the one in the real factory
		gc_ptr<ResourcePoolEventObserver> getEventObserver() const {
			return _realFactory->getEventObserver();
		}

	protected:
		virtual Clone* DoClone() const;

	private:
		gc_ptr<SQLFactory_Impl<TM> > _realFactory;
		databaseType_t resourceTypeCode;
};

} // namespace SQL
} // namespace fatalmind

#endif
