//********************************************************************
//*** ResourcePool/SQL/RTNULL.t.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTNULL.t.cc,v 1.12 2009-06-22 07:58:00 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <memory>
#include "t/main.hh"
#include "config.h"
#include "ResourcePool/ResourcePool.hh"
#include "ResourcePool/SQL/ResourcePool.hh"

#include "testconfig.cc"

std::string getLobTable(RP& p) {
#ifdef RESOURCEPOOL_HAVE_ORACLE
	if (&p == oci9_rp.get()) {
		return "create table fatalmind_lob (id numeric not null primary key, val  clob)";
	}
#endif
	return "create table fatalmind_lob (id numeric not null primary key, val  LONGBLOB)";
}

#include "NULL.t.cc"

void INIT_SQL_NULL() {
	open_pools();
#ifdef RESOURCEPOOL_HAVE_ORACLE
	test_NULL_init(*oci9_rp);
#endif
#ifdef RESOURCEPOOL_HAVE_MYSQL
	test_NULL_init(*mysql3_rp);
#endif
}

void TEST_SQL_OCI9_NULL_insert() {
	test_sql_oci_wrapper(&test_NULL_insert);
}
void TEST_SQL_OCI9_NULL_select() {
	test_sql_oci_wrapper(&test_NULL_select);
}

void TEST_SQL_OCI9_NULL_select_CONSTRAINT() {
	test_sql_oci_wrapper(&test_NULL_select_CONSTRAINT);
}

void TEST_SQL_OCI9_NULL_lob() {
	test_sql_oci_wrapper(&test_NULL_lob);
}
void TEST_SQL_OCI9_NULL_numeric() {
	test_sql_oci_wrapper(&test_NULL_numeric);
}




void TEST_SQL_MYSQL3_NULL_insert() {
	test_sql_mysql_wrapper(&test_NULL_insert);
}

void TEST_SQL_MYSQL3_NULL_select() {
	test_sql_mysql_wrapper(&test_NULL_select);
}

void TEST_SQL_MYSQL3_NULL_select_CONSTRAINT() {
	test_sql_mysql_wrapper(&test_NULL_select_CONSTRAINT);
}

void TEST_SQL_MYSQL3_NULL_lob() {
	test_sql_mysql_wrapper(&test_NULL_lob);
}

void TEST_SQL_MYSQL3_NULL_numeric() {
	test_sql_mysql_wrapper(&test_NULL_numeric);
}


void CLEANUP_SQL_NULL() { 
#ifdef RESOURCEPOOL_HAVE_ORACLE
	test_NULL_cleanup(*oci9_rp);
#endif
#ifdef RESOURCEPOOL_HAVE_MYSQL
	test_NULL_cleanup(*mysql3_rp);
#endif
}
