//********************************************************************
//*** ResourcePool/SQL/RTSQLTransaction.t.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLTransaction.t.cc,v 1.14 2009-06-21 15:51:55 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <memory>
#include "t/main.hh"
#include "config.h"
#include "ResourcePool/ResourcePool.hh"
#include "ResourcePool/SQL/ResourcePool.hh"

#include "testconfig.cc"

#include "SQLTransaction.t.cc"

void test_init()
{
#ifdef RESOURCEPOOL_HAVE_ORACLE
    if (oci9_rp.get() == 0) {
        ResourcePoolOptions opt;
        opt.setMaxExecTry(1);
        oci9_rp.reset(new RP(getFactory("oci9:" + getOracleLogon()), opt));
    }
#endif
#ifdef RESOURCEPOOL_HAVE_MYSQL
    if (mysql3_rp.get() == 0) {
        ResourcePoolOptions opt;
        opt.setMaxExecTry(1);
        mysql3_rp.reset(new RP(getFactory("mysql:" + getMysqlLogon()), opt));
    }
#endif
}

void INIT_SQL_Transaction() 
{
	test_init();
#ifdef RESOURCEPOOL_HAVE_ORACLE
{
	init(*oci9_rp);	
	RP::Commands::SQLExecute init(*oci9_rp, "insert into hirsch (id, ukn) values (?, ?)");
	for (int i = 0; i < 300; ++i) {
		init.bind(0, i);
		init.bind(1, "hirschig");
		oci9_rp->execute(init);
	}
}
#endif
#ifdef RESOURCEPOOL_HAVE_MYSQL_TRANSACTIONS
{
	init(*mysql3_rp);	
	RP::Commands::SQLExecute init(*mysql3_rp, "insert into hirsch (id, ukn) values (?, ?)");
	for (int i = 0; i < 300; ++i) {
		init.bind(0, i);
		init.bind(1, "hirschig");
		mysql3_rp->execute(init);
	}
}
#endif
}

void TEST_SQL_OCI9_Transaction_one()
{
#ifdef RESOURCEPOOL_HAVE_ORACLE
	SQLTransaction_init_me(*oci9_rp);
	trans_test_ok(*oci9_rp, 0);
#else
	throw SkippedException("Oracle support was not enabled in this ResourcePool build");
#endif
}

void TEST_SQL_MYSQL3_Transaction_one()
{
#ifdef RESOURCEPOOL_HAVE_MYSQL_TRANSACTIONS
	SQLTransaction_init_me(*mysql3_rp);
	trans_test_ok(*mysql3_rp, 0);
#else
	throw SkippedException("MySQL transaction support was not enabled in this ResourcePool build");
#endif
}

void TEST_PERF_SQL_OCI9_Transaction_one()
{
#ifdef RESOURCEPOOL_HAVE_ORACLE
	SQLTransaction_init_me(*oci9_rp);
	for (int i = 0; i < 100; ++i) {
		trans_test_ok(*oci9_rp, i);
	}
#else
	throw SkippedException("Oracle support was not enabled in this ResourcePool build");
#endif
}

void TEST_PERF_SQL_MYSQL3_Transaction_one()
{
#ifdef RESOURCEPOOL_HAVE_MYSQL_TRANSACTIONS
	SQLTransaction_init_me(*mysql3_rp);
	for (int i = 0; i < 100; ++i) {
		trans_test_ok(*mysql3_rp, i);
	}
#else
	throw SkippedException("MySQL transaction support was not enabled in this ResourcePool build");
#endif
}

void TEST_PERF_SQL_OCI9_Transaction_one_cached()
{
#ifdef RESOURCEPOOL_HAVE_ORACLE
	SQLTransaction_init_me(*oci9_rp);
	for (int i = 0; i < 100; ++i) {
		trans_test_ok(*oci9_rp, i, true);
	}
#else
	throw SkippedException("Oracle support was not enabled in this ResourcePool build");
#endif
}

void TEST_PERF_SQL_MYSQL3_Transaction_one_cached()
{
#ifdef RESOURCEPOOL_HAVE_MYSQL_TRANSACTIONS
	SQLTransaction_init_me(*mysql3_rp);
	for (int i = 0; i < 100; ++i) {
		trans_test_ok(*mysql3_rp, i, true);
	}
#else
	throw SkippedException("MySQL transaction support was not enabled in this ResourcePool build");
#endif
}

void TEST_SQL_OCI9_Transaction_rec()
{
#ifdef RESOURCEPOOL_HAVE_ORACLE
	SQLTransaction_rec(*oci9_rp);
#else
	throw SkippedException("Oracle support was not enabled in this ResourcePool build");
#endif
}

void TEST_SQL_MYSQL3_Transaction_rec()
{
#ifdef RESOURCEPOOL_HAVE_MYSQL_TRANSACTIONS
	SQLTransaction_rec(*mysql3_rp);
#else
	throw SkippedException("MySQL transaction support was not enabled in this ResourcePool build");
#endif
}

void TEST_SQL_OCI9_Transaction_rec_fail()
{
#ifdef RESOURCEPOOL_HAVE_ORACLE
	SQLTransaction_rec_fail(*oci9_rp);
#else
	throw SkippedException("Oracle support was not enabled in this ResourcePool build");
#endif
}

void TEST_SQL_MYSQL3_Transaction_rec_fail()
{
#ifdef RESOURCEPOOL_HAVE_MYSQL_TRANSACTIONS
	SQLTransaction_rec_fail(*mysql3_rp);
#else
	throw SkippedException("MySQL transaction support was not enabled in this ResourcePool build");
#endif
}

void TEST_SQL_OCI9_Transaction_fail()
{
#ifdef RESOURCEPOOL_HAVE_ORACLE
	SQLTransaction_init_me(*oci9_rp);
	trans_test_fail(*oci9_rp, 0);
#else
	throw SkippedException("Oracle support was not enabled in this ResourcePool build");
#endif
}

void TEST_SQL_MYSQL3_Transaction_fail()
{
#ifdef RESOURCEPOOL_HAVE_MYSQL_TRANSACTIONS
	SQLTransaction_init_me(*mysql3_rp);
	trans_test_fail(*mysql3_rp, 0);
#else
	throw SkippedException("MySQL transaction support was not enabled in this ResourcePool build");
#endif
}

void TEST_PERF_SQL_OCI9_Transaction_fail()
{
#ifdef RESOURCEPOOL_HAVE_ORACLE
	SQLTransaction_init_me(*oci9_rp);
	for (int i = 0; i < 100; ++i) {
		trans_test_fail(*oci9_rp, i);
	}
#else
	throw SkippedException("Oracle support was not enabled in this ResourcePool build");
#endif
}

void TEST_PERF_SQL_MYSQL3_Transaction_fail()
{
#ifdef RESOURCEPOOL_HAVE_MYSQL_TRANSACTIONS
	SQLTransaction_init_me(*mysql3_rp);
	for (int i = 0; i < 100; ++i) {
		trans_test_fail(*mysql3_rp, i);
	}
#else
	throw SkippedException("MySQL transaction support was not enabled in this ResourcePool build");
#endif
}

void TEST_PERF_SQL_OCI9_Transaction_fail_cached()
{
#ifdef RESOURCEPOOL_HAVE_ORACLE
	SQLTransaction_init_me(*oci9_rp);
	for (int i = 0; i < 100; ++i) {
		trans_test_fail(*oci9_rp, i, true);
	}
#else
	throw SkippedException("Oracle support was not enabled in this ResourcePool build");
#endif
}


void TEST_PERF_SQL_MYSQL3_Transaction_fail_cached()
{
#ifdef RESOURCEPOOL_HAVE_MYSQL_TRANSACTIONS
	SQLTransaction_init_me(*mysql3_rp);
	for (int i = 0; i < 100; ++i) {
		trans_test_fail(*mysql3_rp, i, true);
	}
#else
	throw SkippedException("MySQL transaction support was not enabled in this ResourcePool build");
#endif
}


void TEST_SQL_OCI9_Transaction_fail_rec() {
#ifdef RESOURCEPOOL_HAVE_ORACLE
	SQLTransaction_init_me(*oci9_rp);
	trans_test_fail_rec(*oci9_rp, 0);
#else
	throw SkippedException("Oracle support was not enabled in this ResourcePool build");
#endif
}

void TEST_SQL_MYSQL3_Transaction_fail_rec() {
#ifdef RESOURCEPOOL_HAVE_MYSQL_TRANSACTIONS
	SQLTransaction_init_me(*mysql3_rp);
	trans_test_fail_rec(*mysql3_rp, 0);
#else
	throw SkippedException("MySQL transaction support was not enabled in this ResourcePool build");
#endif
}

void TEST_PERF_SQL_OCI9_Transaction_fail_rec() {
#ifdef RESOURCEPOOL_HAVE_ORACLE
	SQLTransaction_init_me(*oci9_rp);
	for (int i = 0; i < 100; ++i) {
		trans_test_fail_rec(*oci9_rp, i);
	}
#else
	throw SkippedException("Oracle support was not enabled in this ResourcePool build");
#endif
}

void TEST_PERF_SQL_MYSQL3_Transaction_fail_rec() {
#ifdef RESOURCEPOOL_HAVE_MYSQL_TRANSACTIONS
	SQLTransaction_init_me(*mysql3_rp);
	for (int i = 0; i < 100; ++i) {
		trans_test_fail_rec(*mysql3_rp, i);
	}
#else
	throw SkippedException("MySQL transaction support was not enabled in this ResourcePool build");
#endif
}

void TEST_PERF_SQL_OCI9_Transaction_fail_rec_ca() {
#ifdef RESOURCEPOOL_HAVE_ORACLE
	SQLTransaction_init_me(*oci9_rp);
	for (int i = 0; i < 100; ++i) {
		trans_test_fail_rec(*oci9_rp, i, true);
	}
#else
	throw SkippedException("Oracle support was not enabled in this ResourcePool build");
#endif
}

void TEST_PERF_SQL_MYSQL3_Transaction_fail_rec_ca() {
#ifdef RESOURCEPOOL_HAVE_MYSQL_TRANSACTIONS
	SQLTransaction_init_me(*mysql3_rp);
	for (int i = 0; i < 100; ++i) {
		trans_test_fail_rec(*mysql3_rp, i, true);
	}
#else
	throw SkippedException("MySQL transaction support was not enabled in this ResourcePool build");
#endif
}
