//********************************************************************
//*** ResourcePool/SQL/RTTypes.t.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTTypes.t.cc,v 1.9 2009-06-22 07:58:00 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <memory>
#include "t/main.hh"
#include "config.h"
#include "ResourcePool/ResourcePool.hh"
#include "ResourcePool/SQL/ResourcePool.hh"

#include "testconfig.cc"


std::string getStringTable(RP& p) {
#ifdef RESOURCEPOOL_HAVE_ORACLE
	if (&p == oci9_rp.get()) {
		return "create table fatalmind_str (id numeric not null primary key, val  varchar2(512))";
	} 
#endif
	return "create table fatalmind_str (id numeric not null primary key, val  varchar(512))";
}

std::string getLobTable(RP& p) {
#ifdef RESOURCEPOOL_HAVE_ORACLE
	if (&p == oci9_rp.get()) {
		return "create table fatalmind_lob (id numeric not null primary key, val  clob)";
	} 
#endif
	return "create table fatalmind_lob (id numeric not null primary key, val  LONGBLOB)";
}
#include "Types.t.cc"

void INIT_SQL_TYPES() {
	open_pools();
#ifdef RESOURCEPOOL_HAVE_ORACLE
	test_Types_init(*oci9_rp);
#endif
#ifdef RESOURCEPOOL_HAVE_MYSQL
	test_Types_init(*mysql3_rp);
#endif
}

void TEST_SQL_OCI9_Types_int() {
	test_sql_oci_wrapper(&test_Types_int);
}
void TEST_SQL_MYSQL_Types_int() {
	test_sql_mysql_wrapper(&test_Types_int);
}

void TEST_SQL_OCI9_Types_float() {
	test_sql_oci_wrapper(&test_Types_float);
}
void TEST_SQL_MYSQL_Types_float() {
	test_sql_mysql_wrapper(&test_Types_float);
}

void TEST_SQL_OCI9_Types_double() {
	test_sql_oci_wrapper(&test_Types_double);
}
void TEST_SQL_MYSQL_Types_double() {
	test_sql_mysql_wrapper(&test_Types_double);
}

void TEST_SQL_OCI9_Types_string() {
	test_sql_oci_wrapper(&test_Types_string);
}

void TEST_SQL_OCI9_Types_crosstype_string() {
	test_sql_oci_wrapper(&test_Types_crosstype_string);
}
void TEST_SQL_MYSQL_Types_crosstype_string() {
	test_sql_mysql_wrapper(&test_Types_crosstype_string);
}

void TEST_SQL_OCI9_Types_string_crosstype() {
	test_sql_oci_wrapper(&test_Types_string_crosstype);
}
void TEST_SQL_MYSQL_Types_string_crosstype() {
	test_sql_mysql_wrapper(&test_Types_string_crosstype);
}

void TEST_SQL_MYSQL_Types_string() {
	test_sql_mysql_wrapper(&test_Types_string);
}

void TEST_SQL_OCI9_Types_lob() {
	test_sql_oci_wrapper(&test_Types_lob);
}
void TEST_SQL_MYSQL_Types_lob() {
	test_sql_mysql_wrapper(&test_Types_lob);
}

void TEST_PERF_SQL_OCI9_Types_lob_4k() {
	test_sql_oci_wrapper(&test_perf_Types_lob_4k);
}
void TEST_PERF_SQL_MYSQL_Types_lob_4k() {
	test_sql_mysql_wrapper(&test_perf_Types_lob_4k);
}

void TEST_PERF_SQL_OCI9_Types_lob_64k() {
	test_sql_oci_wrapper(&test_perf_Types_lob_64k);
}
void TEST_PERF_SQL_MYSQL_Types_lob_64k() {
	test_sql_mysql_wrapper(&test_perf_Types_lob_64k);
}

void TEST_PERF_SQL_OCI9_Types_lob_1024k() {
	test_sql_oci_wrapper(&test_perf_Types_lob_1024k);
}
void TEST_PERF_SQL_MYSQL_Types_lob_1024k() {
	test_sql_mysql_wrapper(&test_perf_Types_lob_1024k);
}

