//********************************************************************
//*** ResourcePool/SQL/SQLStatement.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLStatement.cc,v 1.10 2009-05-04 13:20:43 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#include "ResourcePool/SQL/SQLStatementImpl.hh"
#include "Exceptions/Exception.hh"

namespace fatalmind {
namespace SQL {

SQLStatement::SQLStatement() 
: impl(new SQLStatementImpl())
{
}

SQLStatement::SQLStatement(const std::string& sql)
: impl(new SQLStatementImpl(sql))
{
}

SQLStatement::SQLStatement(const char* sql)
: impl(new SQLStatementImpl(sql))
{
}

SQLStatement::SQLStatement(const SQLStatement& rho)
: impl(rho.impl)
{
}

SQLStatement::SQLStatement(SQLStatementImpl* rho)
: impl()
{
	impl.resetFromExisting(rho);
}

SQLStatement::~SQLStatement()
{
}

unsigned int SQLStatement::getParamCount() const {
	return impl->getParamCount();
}

// TODO: name ref?
std::string SQLStatement::getNameFor(const int pos) const {
	return impl->getNameFor(pos);
}
		
int SQLStatement::getTypeFor(const int pos) const {
	return impl->getTypeFor(pos);
}

const std::string& SQLStatement::getSQL() const {
	return impl->getSQL();	
}
	
const std::string& SQLStatement::getSQLFragment(const int pos) const {
	return impl->getSQLFragment(pos);
}

bool SQLStatement::operator==(const SQLStatement& rho) {
	// check if they point to the same Impl object,
	// if not, still compare the sql string itself
	return (impl.get() == rho.impl.get()) || (*impl == *rho.impl);
}

bool SQLStatement::operator!=(const SQLStatement& rho) {
	return (*impl != *rho.impl);
};

template<class charT, class traits>
std::basic_ostream<charT, traits>& operator<<(std::basic_ostream<charT, traits>& s, const SQLStatement& rho)
{
	s << rho.getSQL();
	return s;
}

// instantiation :(
template
std::basic_ostream<char, std::char_traits<char> >& operator<<(std::basic_ostream<char, std::char_traits<char> >& s, const SQLStatement& rho);

const int SQLStatement::Parameter::UNKNOWN = 0;
const int SQLStatement::Parameter::CHAR    = 1;
const int SQLStatement::Parameter::INT     = 2;
const int SQLStatement::Parameter::FLOAT   = 3;

} // namespace SQL
} // namespace fatalmind
