//********************************************************************
//*** ResourcePool/SQL/SQLStatementImpl.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLStatementImpl.hh,v 1.1 2009-05-04 13:20:43 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef INCLUDE_SQLSTATEMENTIMPL_HH
#define INCLUDE_SQLSTATEMENTIMPL_HH

#include <string>
#include <vector>

#include "Exceptions/Exception.hh"
#include "mm/gc_ptr.hh"
#include "SQLStatement.hh"

namespace fatalmind {
namespace SQL {

class SQLParser; // Fwd

class SQLStatementImpl: public usg_cnt_t<DefaultThreadedModel> { // TODO, TM
	public:
		/** Holds all available information about a placeholder (?) within an SQL string.
		 */
		class Parameter {
			public:
				Parameter() 
				: name()
				, type(SQLStatement::Parameter::UNKNOWN)
				{
				}
			
				std::string getName() const {
					return name;
				}

				int getType() const {
					return type;
				}

			private:
				std::string name;
				int type;

				void setName(const char* const n, int length) {
					name = std::string(n, length);
				}
	
				void setType(const int t) {
					type = t;
				}

				void reset() {
					name.clear();
					type = SQLStatement::Parameter::UNKNOWN;
				}
			
			friend
				class SQLParser;
		};

		SQLStatementImpl();

		SQLStatementImpl(const std::string& sql);
		SQLStatementImpl(const char*);

		void parse();
		
		unsigned int getParamCount() const;
		std::string getNameFor(const int pos) const;
		int getTypeFor(const int pos) const;
		const std::string& getSQL() const;
		const std::string& getSQLFragment(const int pos) const;

		bool operator==(const SQLStatementImpl& rho) {
			return sql == rho.sql;
		};
		bool operator!=(const SQLStatementImpl& rho) {
			return sql != rho.sql;
		};

	private:
		// the parser gives two absolute positions
		// for substr we need the start position and the length
		void addFragment(const int begin, const int end) {
			int length = end - begin;
			fragments.push_back(sql.substr(begin, length));
		}

		void addParam(const Parameter& p) {
			parameters.push_back(p);
		}

		void buildNameMap();

		std::string sql;
		std::vector<std::string>          fragments;
		std::vector<class Parameter>      parameters;
	friend
		class SQLParser;

        template<class charT, class traits>
        friend
                std::basic_ostream<charT, traits>& operator<<(std::basic_ostream<charT, traits>& s, const SQLStatementImpl& rho);
};

} // namespace SQL
} // namespace fatalmind

template<class charT, class traits>
std::basic_ostream<charT, traits>& fatalmind::SQL::operator<<(std::basic_ostream<charT, traits>& s, const fatalmind::SQL::SQLStatementImpl& rho) {
	s << rho.sql;
	return s;
}

#endif
