//********************************************************************
//*** ResourcePool/mysql/Resource.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: Resource.cc,v 1.15 2009-03-02 14:16:45 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#ifndef RESOURCEPOOL_MYSQL_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_CREATEXCEPTION_HH
#include "ResourcePool/CreateException.hh"
#endif

#include <mysql/mysql.h>

#include <string>
#include <sstream>

namespace fatalmind {

mysqlResource::mysqlResource(const std::string& host
	, const std::string& user
	, const std::string& pass
	, const std::string& db
	, gc_ptr<ResourcePoolEventObserver> o
	, unsigned int port 
	, const std::string& sock 
)
: _dbh(0)
, sessionID("N/A")
, observer(o)
{
	Event.PREPARE = observer->registerEvent("PREPARE");
	Event.EXECUTE = observer->registerEvent("EXECUTE");
	Event.FETCH   = observer->registerEvent("FETCH");
	Event.COMMIT  = observer->registerEvent("COMMIT");
	Event.ROLLBACK= observer->registerEvent("ROLLBACK");
	
	const char* c_host = NULL;
	const char* c_pass = NULL;
	const char* c_db   = NULL;

	if ((_dbh = mysql_init(_dbh)) == NULL) {
		throw CreateException(
			"The mysql_init function returned NULL"
		);
	}
	
	if (!host.empty()) {
		c_host = host.c_str();
	}
	if (!pass.empty()) {
		c_pass = pass.c_str();
	}
	if (!db.empty()) {
		c_db = db.c_str();
	}
	

	if (mysql_real_connect(_dbh
			, c_host
			, user.c_str()
			, c_pass
			, c_db
			, port
			, NULL
			, 0) == NULL
	) {
		std::ostringstream ss;
		ss << "Can't connect to database: " << mysql_error(_dbh);
		throw CreateException(ss.str());
	}
}

bool mysqlResource::precheck() 
{
	if (mysql_ping(_dbh)) {
		return false;
	}
	return true;
}

bool mysqlResource::postcheck() 
{
	return true;
}

void mysqlResource::close()
{
	if (_dbh != NULL) {
		mysql_close(_dbh);
		_dbh = NULL;
	}
}

void mysqlResource::fail_close()
{
	if (_dbh != NULL) {
		mysql_close(_dbh);
		_dbh = NULL;
	}
}

MYSQL& mysqlResource::get_plain_resource()
{
	return *_dbh;
}

mysqlResource::~mysqlResource()
{
	close();
}

const std::string& mysqlResource::getSessionID() const {
	std::ostringstream oss;
	oss << mysql_thread_id(_dbh);
	sessionID = oss.str();
	return sessionID;
}

} // namespace fatalmind
