//********************************************************************
//*** ResourcePool/mysql/Resource.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: Resource.hh,v 1.14 2009-05-10 16:09:25 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#ifndef RESOURCEPOOL_MYSQL_RESOURCE_HH
#define RESOURCEPOOL_MYSQL_RESOURCE_HH

#include "ResourcePool/ResourcePoolEventObserver.hh"

#ifndef RESOURCEPOOL_MYSQL_MYSQLFWD_HH
#include "mysqlFwd.hh"
#endif

#include <string>

namespace fatalmind {
namespace mysql {
class FinallyTxDepth;
} // namespace mysql

class mysqlResource {
	public:
		mysqlResource(const std::string& host
			, const std::string& user
			, const std::string& pass
			, const std::string& db
			, gc_ptr<ResourcePoolEventObserver> o
			, unsigned int port = 3306
			, const std::string& sock = ""
		);
		~mysqlResource();
		bool precheck();
		bool postcheck();
		void close();
		void fail_close();
		MYSQL& get_plain_resource();

		int getTxDepth() {
			return _txDepth;
		}

		const std::string& getSessionID() const;
		inline const std::string& getResourceID() const {
			return getSessionID();
		}

		struct Events {
			ResourcePoolEventObserver::event_t PREPARE;
			ResourcePoolEventObserver::event_t EXECUTE;
			ResourcePoolEventObserver::event_t FETCH;
			ResourcePoolEventObserver::event_t COMMIT;
			ResourcePoolEventObserver::event_t ROLLBACK;
		} Event;

		gc_ptr<EventReader> getReader(const ResourcePoolEventObserver::event_t eventid) {
			return observer->getReader(eventid, getResourceID());
		}
	private:
		MYSQL*   _dbh;
	// transaction managment helper function
		int _txDepth;
		int incTxDepth() {
			return _txDepth++;
		}
		void decTxDepth() {
			--_txDepth;
		}
		mutable std::string sessionID;
	
		gc_ptr<ResourcePoolEventObserver> observer;

	friend class mysql::FinallyTxDepth;
};

namespace mysql {
// TODO, use a more generic approach
class FinallyTxDepth {
	public:
		FinallyTxDepth(mysqlResource& a_hand)
		: hand(a_hand)
		{
			hand.incTxDepth();
		}

		~FinallyTxDepth() {
			hand.decTxDepth();
		}
	private:
		mysqlResource& hand;
};
} // namespace mysql

} // namespace fatalmind
#endif
