//********************************************************************
//*** ResourcePool/oracle/SQLSelectRow.t.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelectRow.t.cc,v 1.27 2009-05-11 14:06:05 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "ResourcePool/oracle/ResourcePool.hh"
#include "ResourcePool/oracle/oracleType.hh"
#include "ResourcePool/oracle/oracleRefType.hh"
#include "t/main.hh"

#include "testconfig.cc"
#include "../SQL/SQLSelectRow.t.cc"

#include "cache/Cache.hh"
#include "Resource.cc"
#include "ResourcePool/ResourcePool.cc"
#include "ResourcePool/ResourcePoolEventObserver.hh"
#include "ResourcePool/Command.cc"
#include "SelectListMetaDataCache.hh"

std::auto_ptr<RP> rp;

void INIT_oracle_SQLSelectRow() {
    rp.reset(new RP(getFactory()));
    test_SQLSelectRow_init(*rp);
}

void TEST_oracle_SQLSelectRow_one()
{
	test_SQLSelectRow_one(*rp);
}

void TEST_oracle_SQLSelectRow_none()
{
	test_SQLSelectRow_none(*rp);
}

void TEST_oracle_SQLSelectRow_more()
{
	test_SQLSelectRow_more(*rp);
}

void TEST_oracle_SQLSelectRow_output() {
	test_SQLSelectRow_output(*rp);
}

void TEST_PERF_oracle_SQLSelectRow_massbindout() {
	test_perf_SQLSelectRow_massbindout(*rp);
}

void TEST_PERF_oracle_SQLSelectRow_one_spec_outer_C() {
	test_perf_SQLSelectRow_one_spec_outer(*rp);
}


void TEST_PERF_oracle_SQLSelectRow_one_spec_inner_C() {
	test_perf_SQLSelectRow_one_spec_inner(*rp);
}

void TEST_PERF_oracle_SQLSelectRow_first_one()
{
	test_perf_SQLSelectRow_first_one(*rp);
}

void TEST_PERF_oracle_SQLSelectRow_log_normal() {
	fatalmind::ResourcePoolStreamLogger log(std::cout);
	log.setLogEvents(log.Event.NONE);
	RP::Commands::SQLSelectRow sel(*rp, "Select * from dual");
	for (int i = 0; i <= 1000000; ++i) {
		std::ostringstream os;
		os << "blah;";
		sel.outputoperator(os);
		log.event(log.Event.INCREMENT_POOL, "N/A", os.str());
	}
}

void TEST_PERF_oracle_SQLSelectRow_log_EventReader() {
	fatalmind::ResourcePoolStreamLogger log(std::cout);
	log.setLogEvents(log.Event.NONE);
	RP::Commands::SQLSelectRow sel(*rp, "Select * from dual");
	for (int i = 0; i <= 1000000; ++i) {
		gc_ptr<fatalmind::EventReader> r(log.getReader(log.Event.INCREMENT_POOL, "N/A"));
		(*r) << "blah;" << sel;
	}
}

void CLEANUP_oracle_SQLSelectRow() {
	test_SQLSelectRow_cleanup(*rp);
}

